#include <EEPROM.h>


/********************************************************************/
/*

 
 				        Power Switch
 
 				        Sonar0x11
 
 				 -------------------------   
 				/                         \
 			       /		           \
 		  Sonar0x16   /			            \   Sonar0x12
 		    M3,IR03  /			             \  M2,IR02
     	                    /			              \ 
 			   /			               \
 			  /			                \
 		   	 /			                 \  
 			 \			                 /
 			  \			                /
 		  	   \			               /
 			    \			              /
 		  Sonar0x15  \		  	             / Sonar0x13
 			      \		                    /
 			       \	                   /
 			 	--------------------------  
                                        Sonar0x14
                                        M1,IR01
                                        
 
 */

#include <fuzzy_table.h>
#include <PID_Beta6.h>

#include <PinChangeInt.h>
#include <PinChangeIntConfig.h>

#include <MotorWheel.h>
#include <Omni3WD.h>

#include <SONAR.h>
//--IR 1-D 2-VCC 3-GND
/******************************************/
// SONAR
#define DEBUG
SONAR sonar11(0x11),sonar12(0x12),sonar13(0x13),sonar14(0x14),sonar15(0x15),sonar16(0x16);

unsigned short distBuf[6];
void sonarsUpdate() {
    static unsigned char sonarCurr=1;

    if(sonarCurr==6) sonarCurr=1;
    else ++sonarCurr;
    if(sonarCurr == 1)
    {
      distBuf[5] = sonar16.getDist();
      sonar16.showDat();
      sonar11.trigger();
    }
    else if(sonarCurr == 2)
    {
      distBuf[4] = sonar15.getDist();
      sonar15.showDat();
      sonar12.trigger();
    }
    else if(sonarCurr == 3)
    {
      distBuf[3] = sonar14.getDist();
      sonar14.showDat();
      sonar13.trigger();
    }
    else if(sonarCurr == 4)
    {
      distBuf[2] = sonar13.getDist();
      sonar13.showDat();
      sonar14.trigger();
    }else if(sonarCurr == 5)
    {
      distBuf[1] = sonar12.getDist();
      sonar12.showDat();
      sonar15.trigger();
    }
    else
    {
      distBuf[0] = sonar11.getDist();
      sonar11.showDat();
      sonar16.trigger();
    }
}

/*********************************************/

/*********************************************/
unsigned char IRpin[]={16,17,18};    // Pin16, Pin17, Pin18
bool IRs[3]={false,false,false};

void initIRs() {
    pinMode(IRpin[0],INPUT);
    pinMode(IRpin[1],INPUT);
    pinMode(IRpin[2],INPUT);
}
void checkIRs() {
    for(int i=0;i<3;++i) {
        IRs[i]=digitalRead(IRpin[i]);
       // Serial.print(IRs[i]);
    }
    //Serial.println("");
}
/********************************************/

/*******************************************/
// Wheels

irqISR(irq1,isr1);
MotorWheel wheel1(9,8,6,7,&irq1);        // Pin9:PWM, Pin8:DIR, Pin6:PhaseA, Pin7:PhaseB

irqISR(irq2,isr2);
MotorWheel wheel2(10,11,14,15,&irq2);    // Pin10:PWM, Pin11:DIR, Pin14:PhaseA, Pin15:PhaseB

irqISR(irq3,isr3);
MotorWheel wheel3(3,2,4,5,&irq3);        // Pin3:PWM, Pin2:DIR, Pin4:PhaseA, Pin5:PhaseB
//MotorWheel wheel3(5,4,2,3,&irq3);

Omni3WD Omni(&wheel1,&wheel2,&wheel3);

/******************************************/

/******************************************/
// demo
unsigned long currMillis=0;
void demoWithSensors(unsigned int speedMMPS,unsigned int distance,unsigned int ms) {   
    if(millis()-currMillis>SONAR::duration) {
        currMillis=millis();
        sonarsUpdate();
    }   
    checkIRs();
   
   if(IRs[1] || IRs[2]) {
       Omni.setCarBackoff(speedMMPS);
       Omni.delayMS(ms);
       if(IRs[1]) {
           Omni.setCarRotateLeft(speedMMPS);
       } else Omni.setCarRotateRight(speedMMPS);
       Omni.delayMS(ms);
   } else if(distBuf[1]<distance || distBuf[2]<distance) {
        Omni.setCarLeft(speedMMPS);
    } else if(distBuf[4]<distance || distBuf[5]<distance) {
        Omni.setCarRight(speedMMPS);
    } else if(distBuf[0]<distance || distBuf[3]<distance || IRs[0]) {
        Omni.setCarRotateRight(speedMMPS);
    } else {
        Omni.setCarAdvance(speedMMPS);
    }
    
    Omni.PIDRegulate();
    if(millis()%100==0) Omni.debugger();
}

/*****************************************/
// setup()
void setup() {
    TCCR1B=TCCR1B&0xf8|0x01;    // Pin9,Pin10 PWM 31250Hz
    TCCR2B=TCCR2B&0xf8|0x01;    // Pin3,Pin11 PWM 31250Hz
  
    SONAR::init(13);    // Pin13 as RW Control
    initIRs();
    
    Omni.PIDEnable(0.26,0.02,0,10);
   Serial.begin(9600); 
}

/****************************************/
// loop()
void loop() {
    demoWithSensors(80,20,300);
   //Omni.demoActions(100,3000,500);
}

